VERSION 5.00
Begin VB.Form frmMain 
   AutoRedraw      =   -1  'True
   BackColor       =   &H00000000&
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Hold space to accelerate object CCW"
   ClientHeight    =   4500
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   4500
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   300
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   300
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Private Declare Function GetTickCount Lib "kernel32" () As Long

Const PI = 3.14159
Const GRAVITY = 0.5
Const FRICTION = 0.1
Const LOOP_RADIUS = 100
Const OBJECT_RADIUS = 10
Const OBJECT_ACCEL = 0.5
Const MS_DELAY = 25

Dim mblnRunning As Boolean
Dim mblnKeyDown As Boolean
Dim mlngTimer As Long

Dim msngXCoord As Single
Dim msngYCoord As Single
Dim msngSpeed As Single
Dim msngDirection As Single

Private Sub Form_Load()

    'Show the form
    Me.Show
    
    'Set some initial values
    msngXCoord = 0
    msngYCoord = LOOP_RADIUS - OBJECT_RADIUS
    msngDirection = PI / 2
    msngSpeed = 18 'Some arbitrary inital speed

    'Start the main loop running
    mblnRunning = True
    Do While mblnRunning
        'Check if we've waited for the appropriate number of milliseconds
        If mlngTimer + MS_DELAY <= GetTickCount() Then
            mlngTimer = GetTickCount()  'Reset the timer variable
            Physics
            RenderScene
        End If
        'Allow other events to occur
        DoEvents
    Loop

End Sub

Private Sub Physics()

Dim sngMagNormal As Single
Dim sngDirNormal As Single
Dim sngMagFriction As Single
Dim sngDirFriction As Single
Dim sngMagCentripetal As Single
Dim sngDirCentripetal As Single
Dim sngMagTemp As Single
Dim sngDirTemp As Single
Dim sngAngle As Single

    'When spacebar is pressed, accelerate the object counterclockwise
    If mblnKeyDown Then
        sngAngle = FindAngle(msngXCoord, -msngYCoord) - PI / 2
        AddVectors msngSpeed, msngDirection, OBJECT_ACCEL, sngAngle, msngSpeed, msngDirection
    End If

    'Check for existance of normal force
    If (msngXCoord + msngSpeed * Sin(msngDirection)) ^ 2 + (msngYCoord - msngSpeed * Cos(msngDirection)) ^ 2 > (LOOP_RADIUS - OBJECT_RADIUS) ^ 2 Then
        'Calc the magnitude of the centripetal force
        sngMagCentripetal = msngSpeed ^ 2 / (LOOP_RADIUS - OBJECT_RADIUS)
        'Find the direction of the centripetal force
        sngDirCentripetal = FindAngle(msngXCoord, -msngYCoord)
        'Add the force due to gravity to find maximal normal force
        AddVectors sngMagCentripetal, sngDirCentripetal, GRAVITY, PI, sngMagNormal, sngDirNormal
        'Find the component of this maximal normal force that is currently acting
        sngMagNormal = sngMagNormal * Cos(sngDirNormal - FindAngle(msngXCoord, -msngYCoord))
        If sngMagNormal < 0 Then sngMagNormal = 0
        'Set the normal force's true direction
        sngDirNormal = FindAngle(msngXCoord, -msngYCoord) + PI
        'Calc the magnitude of friction
        sngMagFriction = sngMagNormal * FRICTION
        'Find the direction of the force due to friction
        sngDirFriction = msngDirection + PI
    End If
    
    'Add forces
    AddVectors sngMagNormal, sngDirNormal, sngMagFriction, sngDirFriction, sngMagTemp, sngDirTemp
    AddVectors sngMagTemp, sngDirTemp, GRAVITY, PI, sngMagTemp, sngDirTemp
    
    'Apply forces
    AddVectors sngMagTemp, sngDirTemp, msngSpeed, msngDirection, msngSpeed, msngDirection

    'Adjust coordinates w.r.t. speed
    msngXCoord = msngXCoord + msngSpeed * Sin(msngDirection)
    msngYCoord = msngYCoord - msngSpeed * Cos(msngDirection)
    
    'Ensure we 're still within the loop
    If msngXCoord ^ 2 + msngYCoord ^ 2 > (LOOP_RADIUS - OBJECT_RADIUS) ^ 2 Then
        sngAngle = FindAngle(msngXCoord, msngYCoord)
        msngXCoord = (LOOP_RADIUS - OBJECT_RADIUS) * Sin(sngAngle)
        msngYCoord = (LOOP_RADIUS - OBJECT_RADIUS) * Cos(sngAngle)
    End If

End Sub

Private Sub Form_KeyDown(KeyCode As Integer, Shift As Integer)

Dim sngAngle As Single

    'Check for spacebar
    If KeyCode = vbKeySpace Then mblnKeyDown = True

End Sub

Private Sub Form_KeyUp(KeyCode As Integer, Shift As Integer)

    'Check for spacebar release
    If KeyCode = vbKeySpace Then mblnKeyDown = False

End Sub

Private Sub RenderScene()

    'Clear the form
    Me.Cls

    'Draw the loop
    Circle (ScaleWidth \ 2, ScaleHeight \ 2), LOOP_RADIUS, vbWhite
    
    'Draw the object
    Circle (ScaleWidth \ 2 + msngXCoord, ScaleHeight \ 2 + msngYCoord), OBJECT_RADIUS, vbRed

End Sub

Private Sub AddVectors(ByVal sngMag1, ByVal sngDir1, ByVal sngMag2, ByVal sngDir2, ByRef sngMagResult, ByRef sngDirResult)

Dim sngXComp As Single
Dim sngYComp As Single

    'Determine the X and Y components of the resultant vector
    sngXComp = sngMag1 * Sin(sngDir1) + sngMag2 * Sin(sngDir2)
    sngYComp = sngMag1 * Cos(sngDir1) + sngMag2 * Cos(sngDir2)
    'Determine the resultant magnitude
    sngMagResult = Sqr(sngXComp ^ 2 + sngYComp ^ 2)
    'Calculate the resultant heading
    sngDirResult = FindAngle(sngXComp, sngYComp)

End Sub

Private Function FindAngle(ByRef sngX, ByRef sngY) As Single

    'Determine the angle to the given set of coords
    If Sgn(sngY) > 0 Then FindAngle = Atn(sngX / sngY)
    If Sgn(sngY) < 0 Then FindAngle = Atn(sngX / sngY) + PI
    If sngY = 0 And Sgn(sngX) < 0 Then FindAngle = 3 * PI / 2
    If sngY = 0 And Sgn(sngX) > 0 Then FindAngle = PI / 2

End Function

Private Sub Form_Unload(Cancel As Integer)

    'Terminate the main loop
    mblnRunning = False

End Sub
